clear
use data_files/trocolen.dta

// compute smoothed pdf and cdf
//  download -akdensity- from SSC: ssc install akdensity 
  range x 200 600 100
  label var x "Length" // label vars for graphs 
  akdensity length , nograph at(x) gen(fx)  cdf(Fx) // note 'nograph' option
  cumul length , gen(ecdf) 
    
// prepare graph layout -- use local for clarity  
loc gropts ///
		 scheme(canazei)          ///   sober scheme
		 yscale(range(-0.0005 0.008) axis(1))  ///   fix range of axis 1
		 ylabel(0(.002)0.008 , axis(1))        ///
		 xtitle("Coral trout length (in mm.)") ///
		 ytitle("Adaptive kernel PDF estimate" , axis(1)) ///
		 ytitle("Adaptive kernel CDF estimate" , axis(2)) 

// add markers for individual points
gen to = - 0.05  // position markers
loc spikes  dropline to length , msymbol(none) yaxis(2) 

// bring it all together: overlay components (assign relevant y-axis)
twoway ///
	(line fx x) ///
	(line Fx x, yaxis(2)) ///
	(line ecdf length, connect(stairstep) sort yaxis(2) ) ///
	(`spikes') ///
	, `gropts'  title(Example 1 but with (horrible) custom scheme) ///
	saving(example1_canazei.gph, replace) 

	
exit





