use data_files/psell3_canazei_08.dta ,  clear  // NB: this file can not be distributed

// compute coordinates
inequal7 y [aw=pwgt] 
loc gini = r(gini)
glcurve y [aw=pwgt] , pvar(py) glvar(ly) lorenz nograph
lab var py "Cumulative population share"
sort py

// graph options
loc gropts  ///
	plotregion(style(outline) lcolor(green) margin(zero))     ///
	graphregion(style(none) color(white))    ///
	aspectratio(1) ///
	scheme(s1color) legend(off) ///
	xtitle("Cumulative population share") 	///
	ytitle("Lorenz ordinates") ///
	title("Lorenz curve") ///
	ylabel( , angle(horizontal) grid ) ///
	xlabel( 0(.2)1 , grid)
	
loc clopts1  ///
	lpattern(solid) lcolor("51 0 102") lwidth(thick)

loc aropts ///
	color("51 0 102*0.2")

loc addtext ///
	text(0.7 0.3 "Gini coefficient: `: di %4.3f `gini''")

// add labels for top axis to show income levels
foreach p of numlist .02 .2(.2).8 .98 {
	_pctile y [aw=pwgt] , percentile(`=`p'*100')
	loc xlabs 	`xlabs'  	`p' "EUR `: di %4.0f round(r(r1)/12,100) '"
}
loc gropts  `gropts'  xlabel( `xlabs' , angle(30) labsize(small) axis(2) )  xtitle("", axis(2)) 
		
	
	
// bring it all together
twoway ///
	(rarea py ly py ,  `aropts') ///
	(line ly py,  `clopts1')  			  ///
	(function equality = x, range(0 1) xaxis(2) )	  ///
	, `gropts' `addtext' saving(example2.gph, replace)  

	exit
