clear
// generate data: score on 20 of two 'individuals' on 5 subjects
 input id k val
 1 1 10 
 1 2 12
 1 3 15
 1 4 4
 1 5 5
 2 1 18 
 2 2 20 
 2 3 12
 2 4 7
 2 5 12
end
lab def labk 1 "Math" 2 "History" 3 "Geography" 4 "Economics" 5 "Literature"
lab val k labk


// clear the decks:
loc gropts ///
	yscale(off) xscale(off) legend(off) graphregion(style(none) color(white)) plotregion(style(none))   ///
	aspectratio(1) 

// color of structure:
loc struct ///
	lpattern(solid) lcolor(gs12) lwidth(thin)	
	
// locate coordinates of the 10 scores
//   - divide 360 degrees in 5 slices starting at 90: 90, 90+72, ... 
//   - convert degrees into radius (Stata reads radius scale)
//   - find coordinates as sine and cosine of the different angles
//   - slide them down towards origin according to fraction of 20
gen xpos = (val/20) * cos( (90 + (k-1)*(360/5)) *_pi/180 )
gen ypos = (val/20) * sin( (90 + (k-1)*(360/5)) *_pi/180 )

// create a path through the 5 score locations 
bys id (k) : gen yposo = ypos[_n-1]              //origin is location of previous subject 
bys id (k) : replace yposo = ypos[_N] if _n==1   // ... and close the loop
bys id (k) : gen xposo = xpos[_n-1]
bys id (k) : replace xposo = xpos[_N] if _n==1

//	(pccapsym ypos xpos yposo xposo if id==1) 
//	(pccapsym ypos xpos yposo xposo  if id==2) 


// background radar structure: generate points for markers along the 5 rays (at 4, 8, 12, 16 and 20)
// same logic as before
gen orig = 0
loc j 0
foreach i of numlist .2(.2)1 {
	gen xref`++j' = `i' * cos( (90 + (k-1)*(360/5)) *_pi/180 )  if id==1 
	gen yref`j'   = `i' * sin( (90 + (k-1)*(360/5)) *_pi/180 )  if id==1
    loc txtlabs `"`txtlabs' text(`i' 0.04 "`=`i'*20'") "'    
	bys id (k) : gen xrefo`j' = xref`j'[_n-1]
	bys id (k) : replace xrefo`j' = xref`j'[_N] if _n==1
	bys id (k) : gen yrefo`j' = yref`j'[_n-1]
	bys id (k) : replace yrefo`j' = yref`j'[_N] if _n==1

	loc pcspikes `pcspikes' (pcspike yref`j' xref`j' yrefo`j' xrefo`j'  if id==1 , `struct' ) 
	}

* locate position of text label for subject field
gen xhead = xref5 * 1.1
gen yhead = yref5 * 1.1

* bring it all together:
tw ///
	`pcspikes'  ///  the 5 contour lines
	(pcspike yref5 xref5 orig orig  if id==1, `struct' ) /// rays from origin
	(scatter yhead xhead if id==1, msy(i) mlabel(k) mlabpos(0) ) /// labels
	(pccapsym ypos xpos yposo xposo if id==1) /// contour of id==1
	(pccapsym ypos xpos yposo xposo  if id==2) /// contour of id==2
	, `txtlabs' `gropts' saving(example4.gph, replace)

// Alternative with rounded 'radar' look using hollowed symbols from origin:
tw ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(120)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(95)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(70)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(48)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(23)  ) ///
	(pcspike yref5 xref5 orig orig  if id==1, `struct'  ) ///
	(scatter yhead xhead if id==1, msy(i) mlabel(k) mlabpos(0) ) ///
	(pccapsym ypos xpos yposo xposo if id==1) ///
	(pccapsym ypos xpos yposo xposo  if id==2) ///
	, `txtlabs' `gropts' saving(example4b.gph,replace) 


// Alternative with area shading: this is done using scatteri and recast(area)
  sort id k
  loc immediate1 `=ypos[1]' `=xpos[1]' `=ypos[2]' `=xpos[2]' `=ypos[3]' `=xpos[3]' `=ypos[4]' `=xpos[4]' `=ypos[5]' `=xpos[5]' `=ypos[1]' `=xpos[1]' 
  gsort -id k
  loc immediate2 `=ypos[1]' `=xpos[1]' `=ypos[2]' `=xpos[2]' `=ypos[3]' `=xpos[3]' `=ypos[4]' `=xpos[4]' `=ypos[5]' `=xpos[5]' `=ypos[1]' `=xpos[1]' 
      	
tw ///
	(scatteri `immediate2' ,recast(area) ) ///
	(scatteri `immediate1' ,recast(area) ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(120)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(95)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(70)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(48)  ) ///
	(scatter orig orig if id==1 & k==1, msy(oh) mcolor(gs12) msize(23)  ) ///
	(pcspike yref5 xref5 orig orig  if id==1, `struct'  ) ///
	(scatter yhead xhead if id==1, msy(i) mlabel(k) mlabpos(0) ) ///
	(pccapsym ypos xpos yposo xposo  if id==1) ///
	(pccapsym ypos xpos yposo xposo  if id==2) ///
	, `txtlabs' `gropts' saving(example4c.gph,replace) 	
exit
	
	
	
	
	